@@ -7,9 +7,19 @@
 #' package.
 #' 
 #' @inheritParams brm
-#' @param data A list of data.frames each of which will be used to fit a
+#' @param data A \emph{list} of data.frames each of which will be used to fit a
 #'   separate model. Alternatively, a \code{mids} object from the \pkg{mice}
 #'   package.
+#' @param data2 A \emph{list} of named lists each of which will be used to fit a
+#'   separate model. Each of the named lists contains objects representing data
+#'   which cannot be passed via argument \code{data} (see \code{\link{brm}} for
+#'   examples). The length of the outer list should match the length of the list
+#'   passed to the \code{data} argument.
+#' @param recompile Logical, indicating whether the Stan model should be
+#'   recompiled for every imputed data set. Defaults to \code{FALSE}. If
+#'   \code{NULL}, \code{brm_multiple} tries to figure out internally, if recompilation
+#'   is necessary, for example because data-dependent priors have changed.
+#'   Using the default of no recompilation should be fine in most cases.
 #' @param combine Logical; Indicates if the fitted models should be combined
 #'   into a single fitted model object via \code{\link{combine_models}}.
 #'   Defaults to \code{TRUE}.
@@ -59,13 +69,19 @@
 #' 
 #' @export
 brm_multiple <- function(formula, data, family = gaussian(), prior = NULL, 
-                         autocor = NULL, cov_ranef = NULL, 
+                         data2 = NULL, autocor = NULL, cov_ranef = NULL, 
                          sample_prior = c("no", "yes", "only"), 
                          sparse = NULL, knots = NULL, stanvars = NULL,
-                         stan_funs = NULL, combine = TRUE, fit = NA,
-                         seed = NA, file = NULL, ...) {
+                         stan_funs = NULL, silent = 1, recompile = FALSE,
+                         combine = TRUE, fit = NA, seed = NA, 
+                         file = NULL, file_refit = "never", ...) {
+  
   combine <- as_one_logical(combine)
+  file_refit <- match.arg(file_refit, c("never", "on_change"))
   if (!is.null(file)) {
+    if(file_refit == "on_change") {
+      stop2("file_refit = 'on_change' is not supported for brm_multiple yet.")
+    }
     # optionally load saved model object
     if (!combine) {
       stop2("Cannot use 'file' if 'combine' is FALSE.")
@@ -76,24 +92,37 @@ brm_multiple <- function(formula, data, family = gaussian(), prior = NULL,
     }
   }
   
-  data.name <- substitute_name(data)
+  silent <- validate_silent(silent)
+  recompile <- as_one_logical(recompile)
+  data_name <- substitute_name(data)
   if (inherits(data, "mids")) {
     require_package("mice", version = "3.0.0")
     data <- lapply(seq_len(data$m), mice::complete, data = data)
   } else if (!is_data_list(data)) {
     stop2("'data' must be a list of data.frames.")
   }
+  if (!is.null(data2)) {
+    if (!is_data2_list(data2)) {
+      stop2("'data2' must be a list of named lists.")
+    }
+    if (length(data2) != length(data)) {
+      stop2("'data2' must have the same length as 'data'.")
+    }
+  }
   
   if (is.brmsfit(fit)) {
     # avoid complications when updating the model
     class(fit) <- setdiff(class(fit), "brmsfit_multiple")
   } else {
     args <- nlist(
-      formula, data = data[[1]], family, prior, autocor, cov_ranef,
-      sample_prior, sparse, knots, stanvars, stan_funs, seed, ...
+      formula, data = data[[1]], family, prior, data2 = data2[[1]], 
+      autocor, cov_ranef, sample_prior, sparse, knots, stanvars, 
+      stan_funs, silent, seed, ...
     )
     args$chains <- 0
-    message("Compiling the C++ model")
+    if (silent < 2) {
+      message("Compiling the C++ model")
+    }
     fit <- suppressMessages(do_call(brm, args))
   }
   
@@ -107,12 +136,15 @@ brm_multiple <- function(formula, data, family = gaussian(), prior = NULL,
   fits <- futures <- rhats <- vector("list", length(data))
   for (i in seq_along(data)) {
     futures[[i]] <- future::future(
-      update(fit, newdata = data[[i]], recompile = FALSE, ...),
-      packages = "brms"
+      update(fit, newdata = data[[i]], data2 = data2[[i]],
+             recompile = recompile, silent = silent, ...),
+      packages = "brms", seed = TRUE
     )
   }
   for (i in seq_along(data)) {
-    message("Fitting imputed model ", i)
+    if (silent < 2) {
+      message("Fitting imputed model ", i) 
+    }
     fits[[i]] <- future::value(futures[[i]]) 
     rhats[[i]] <- data.frame(as.list(rhat(fits[[i]])))
     if (any(rhats[[i]] > 1.1, na.rm = TRUE)) {
@@ -121,7 +153,7 @@ brm_multiple <- function(formula, data, family = gaussian(), prior = NULL,
   }
   if (combine) {
     fits <- combine_models(mlist = fits, check_data = FALSE)
-    fits$data.name <- data.name
+    attr(fits$data, "data_name") <- data_name
     fits$rhats <- do_call(rbind, rhats)
     class(fits) <- c("brmsfit_multiple", class(fits))
   }
@@ -134,7 +166,7 @@ brm_multiple <- function(formula, data, family = gaussian(), prior = NULL,
 #' Combine Models fitted with \pkg{brms}
 #' 
 #' Combine multiple \code{brmsfit} objects, which fitted the same model.
-#' This is usefuly for instance when having manually run models in parallel.
+#' This is usefully for instance when having manually run models in parallel.
 #' 
 #' @param ... One or more \code{brmsfit} objects.
 #' @param mlist Optional list of one or more \code{brmsfit} objects.
@@ -184,11 +216,16 @@ combine_models <- function(..., mlist = NULL, check_data = TRUE) {
   models[[1]]
 }
 
-# validity check for data input of 'brm_multiple'
+# validity check for 'data' input of 'brm_multiple'
 is_data_list <- function(x) {
   is.list(x) && is.vector(x)
 }
 
+# validity check for 'data2' input of 'brm_multiple'
+is_data2_list <- function(x) {
+  is.list(x) && all(ulapply(x, function(y) is.list(y) && is_named(y)))
+}
+
 warn_brmsfit_multiple <- function(x, newdata = NULL) {
   if (is.brmsfit_multiple(x) && is.null(newdata)) {
     warning2(
