@@ -635,24 +635,32 @@ split_dots <- function(x, ..., model_names = NULL, other = TRUE) {
 
 # reorder observations to be in the initial user-defined order
 # currently only relevant for autocorrelation models 
-# @param eta 'nsamples' x 'nobs' matrix
+# @param eta 'nsamples' x 'nobs' matrix or array
 # @param old_order optional vector to retrieve the initial data order
 # @param sort keep the new order as defined by the time-series?
 # @return the 'eta' matrix with possibly reordered columns
 reorder_obs <- function(eta, old_order = NULL, sort = FALSE) {
   stopifnot(length(dim(eta)) %in% c(2L, 3L))
-  if (!is.null(old_order) && !sort) {
-    if (isTRUE(length(old_order) == ncol(eta))) {
-      if (length(dim(eta)) == 3L) {
-        eta <- eta[, old_order, , drop = FALSE]   
-      } else {
-        eta <- eta[, old_order, drop = FALSE]   
-      }
-    } else {
-      warning2("Cannot recover the original observation order.")
-    }
+  if (is.null(old_order) || sort) {
+    return(eta)
   }
-  eta
+  stopifnot(length(old_order) == NCOL(eta))
+  p(eta, old_order, row = FALSE)
+}
+
+# update .MISC environment of the stanfit object
+# allows to call log_prob and other C++ using methods
+# on objects not created in the current R session
+update_misc_env <- function(x) {
+  stopifnot(is.brmsfit(x))
+  if (!isTRUE(x$backend == "rstan")) {
+    # .MISC env is only relevant for rstan
+    return(x)
+  }
+  # TODO: detect when updating .MISC is not required
+  # TODO: find a more efficient way to update .MISC
+  x$fit@.MISC <- suppressMessages(brm(fit = x, chains = 0))$fit@.MISC
+  x
 }
 
 # extract argument names of a post-processing method
@@ -667,11 +675,99 @@ arg_names <- function(method) {
   out
 }
 
+#' Check if cached fit can be used.
+#' 
+#' Checks whether a given cached fit can be used without refitting when 
+#' \code{file_refit = "on_change"} is used.
+#' This function is internal and exposed only to facilitate debugging problems 
+#' with cached fits. The function may change or be removed in future versions
+#' and scripts should not use it.
+#' 
+#' @param fit Old \code{brmsfit} object (e.g., loaded from file).
+#' @param sdata New Stan data (result of a call to \code{\link{make_standata}}).
+#'   Pass \code{NULL} to avoid data check.
+#' @param scode New Stan code (result of a call to \code{\link{make_stancode}}).
+#'   Pass \code{NULL} to avoid code check.
+#' @param algorithm New algorithm. Pass \code{NULL} to avoid algorithm check.
+#' @param silent Logical. If \code{TRUE}, no messages will be given.
+#' @param verbose Logical. If \code{TRUE} detailed report of the differences 
+#'   is printed to the console.
+#' @return A boolean indicating whether a refit is needed.
+#' 
+#' @details 
+#' Use with \code{verbose = TRUE} to get additional info on how the stored
+#' fit differs from the given data and code.
+#' 
+#' @export
+#' @keywords internal
+brmsfit_needs_refit <- function(fit, sdata = NULL, scode = NULL, 
+                                algorithm = NULL, silent = FALSE, 
+                                verbose = FALSE) {
+  stopifnot(is.brmsfit(fit))
+  silent <- as_one_logical(silent)
+  verbose <- as_one_logical(verbose)
+  if (!is.null(scode)) {
+    scode <- as_one_character(scode)
+    cached_scode <- stancode(fit)
+  }
+  if (!is.null(sdata)) {
+    stopifnot(is.list(sdata))
+    cached_sdata <- standata(fit)
+  }
+  if (!is.null(algorithm)) {
+    algorithm <- as_one_character(algorithm)
+    stopifnot(!is.null(fit$algorithm))
+  }
+  
+  refit <- FALSE
+  if (!is.null(scode)) {
+    if (normalize_stancode(scode) != normalize_stancode(cached_scode)) {
+      if (!silent) {
+        message("Stan code changed beyond whitespace/comments.")
+        if (verbose) {
+          require_package("diffobj")
+          print(diffobj::diffChr(scode, cached_scode, format = "ansi8"))
+        }
+      }
+      refit <- TRUE
+    }
+  }
+  if (!is.null(sdata)) {
+    sdata_equality <- all.equal(sdata, cached_sdata, check.attributes = FALSE)
+    if (!isTRUE(sdata_equality)) {
+      if (!silent) {
+        message("The processed data for Stan changed.\n")
+        if (verbose) {
+          print(sdata_equality)
+        }
+      }
+      refit <- TRUE
+    }
+  }
+  if (!is.null(algorithm)) {
+    if (algorithm != fit$algorithm) {
+      if (!silent) {
+        message("Algorithm changed from '", fit$algorithm, 
+                "' to '", algorithm, "'.\n")
+      }
+      refit <- TRUE
+    }
+  }
+  refit
+}
+
 # read a brmsfit object from a file
 # @param file path to an rds file
 # @return a brmsfit object or NULL
 read_brmsfit <- function(file) {
   file <- check_brmsfit_file(file)
+  dir <- dirname(file)
+  if (!dir.exists(dir)) {
+    stop2(
+      "The directory '", dir, "' does not exist. Please choose an ",
+      "existing directory where the model can be saved after fitting."
+    )
+  }
   x <- suppressWarnings(try(readRDS(file), silent = TRUE))
   if (!is(x, "try-error")) {
     if (!is.brmsfit(x)) {
