@@ -51,6 +51,10 @@
 #'   or alternatively a \code{\link{cor_brms}} object (deprecated).
 #'   If \code{autocor} is specified in \code{brmsformula}, it will 
 #'   overwrite the value specified in other functions.
+#' @param unused An optional \code{formula} which contains variables
+#'   that are unused in the model but should still be stored in the
+#'   model's data frame. This can be useful, for example,
+#'   if those variables are required for post-processing the model.
 #' 
 #' @return An object of class \code{brmsformula}, which
 #'   is essentially a \code{list} containing all model
@@ -248,7 +252,7 @@
 #'   to \code{TRUE}.
 #'   
 #'   For log-linear models such as poisson models, \code{rate} may be used
-#'   in the \code{aterms} part to specify the denomintor of a response that
+#'   in the \code{aterms} part to specify the denominator of a response that
 #'   is expressed as a rate. The numerator is given by the actual response
 #'   variable and has a distribution according to the family as usual. Using
 #'   \code{rate(denom)} is equivalent to adding \code{offset(log(denom))} to
@@ -316,7 +320,7 @@
 #'   the variable passed to \code{dec} might also be a character vector 
 #'   consisting of \code{'lower'} and \code{'upper'}.
 #'
-#'   For custom families, it is possible to pass an abitrary number of real and
+#'   For custom families, it is possible to pass an arbitrary number of real and
 #'   integer vectors via the addition terms \code{vreal} and \code{vint},
 #'   respectively. An example is provided in
 #'   \code{vignette('brms_customfamilies')}.
@@ -638,7 +642,7 @@
 brmsformula <- function(formula, ..., flist = NULL, family = NULL,
                         autocor = NULL, nl = NULL, loop = NULL, 
                         center = NULL, cmc = NULL, sparse = NULL,
-                        decomp = NULL) {
+                        decomp = NULL, unused = NULL) {
   if (is.brmsformula(formula)) {
     out <- formula
   } else {
@@ -722,6 +726,9 @@ brmsformula <- function(formula, ..., flist = NULL, family = NULL,
   if (!is.null(decomp)) {
     attr(out$formula, "decomp") <- match.arg(decomp, decomp_opts())
   }
+  if (!is.null(unused)) {
+    attr(out$formula, "unused") <- as.formula(unused)
+  }
   if (!is.null(autocor)) {
     attr(out$formula, "autocor") <- validate_autocor(autocor)
   } else if (!is.null(out$autocor)) {
@@ -1020,8 +1027,10 @@ allow_rescor <- function(x) {
   parts <- if (is.mvbrmsformula(x)) x$forms else x$terms 
   families <- lapply(parts, "[[", "family")
   has_rescor <- ulapply(families, has_rescor)
+  is_mixture <- ulapply(families, is.mixfamily)
   family_names <- ulapply(families, "[[", "family")
-  all(has_rescor) && length(unique(family_names)) == 1L
+  all(has_rescor) && !any(is_mixture) &&
+    length(unique(family_names)) == 1L
 }
 
 #' @rdname brmsformula-helpers
@@ -1071,6 +1080,9 @@ plus_brmsformula <- function(e1, e2) {
     e1 <- mvbf(e1, e2)
   } else if (inherits(e2, "setrescor")) {
     stop2("Setting 'rescor' is only possible in multivariate models.")
+  } else if (is.ac_term(e2)) {
+    stop2("Autocorrelation terms can only be specified on the right-hand ",
+          "side of a formula, not added to a 'brmsformula' object.")
   } else if (!is.null(e2)) {
     e1 <- bf(e1, e2)
   }
@@ -1093,6 +1105,9 @@ plus_mvbrmsformula <- function(e1, e2) {
     e1$mecor <- e2[1]
   } else if (is.brmsformula(e2)) {
     e1 <- mvbf(e1, e2)
+  } else if (is.ac_term(e2)) {
+    stop2("Autocorrelation terms can only be specified on the right-hand ",
+          "side of a formula, not added to a 'mvbrmsformula' object.")
   } else if (!is.null(e2)) {
     resp <- attr(e2, "resp", TRUE)
     if (is.null(resp)) {
@@ -1383,7 +1398,7 @@ validate_formula.mvbrmsformula <- function(
       warning2(
         "In the future, 'rescor' will be set to FALSE by default for ", 
         "all models. It is thus recommended to explicitely set ",
-        "'rescor' via 'set_recor' instead of using the default."
+        "'rescor' via 'set_rescor' instead of using the default."
       ) 
     }
   }
