@@ -61,7 +61,8 @@ data_predictor.brmsterms <- function(x, data, data2, prior, ranef,
 data_predictor.btl <- function(x, data, ranef = empty_ranef(), 
                                prior = brmsprior(), data2 = list(),
                                basis = NULL, ...) {
-  c(data_fe(x, data),
+  out <- c(
+    data_fe(x, data),
     data_sp(x, data, data2 = data2, prior = prior, basis = basis$sp),
     data_re(x, data, ranef = ranef),
     data_cs(x, data),
@@ -69,9 +70,10 @@ data_predictor.btl <- function(x, data, ranef = empty_ranef(),
     data_gp(x, data, basis = basis$gp),
     data_ac(x, data, data2 = data2, basis = basis$ac),
     data_offset(x, data),
-    data_bhaz(x, data, data2 = data2, prior = prior, basis = basis$bhaz),
-    data_prior(x, data, prior = prior)
+    data_bhaz(x, data, data2 = data2, prior = prior, basis = basis$bhaz)
   )
+  c(out) <- data_prior(x, data, prior = prior, sdata = out)
+  out
 }
 
 # prepare data for non-linear parameters for use in Stan
@@ -110,7 +112,7 @@ data_sm <- function(bterms, data, basis = NULL) {
     knots <- get_knots(data)
     basis <- named_list(smterms)
     for (i in seq_along(smterms)) {
-      # the spline penality has changed in 2.8.7 (#646)
+      # the spline penalty has changed in 2.8.7 (#646)
       diagonal.penalty <- !require_old_default("2.8.7")
       basis[[i]] <- smoothCon(
         eval2(smterms[i]), data = data, 
@@ -232,7 +234,6 @@ data_gr_local <- function(bterms, data, ranef) {
     levels <- attr(ranef, "levels")[[group]]
     if (id_ranef$gtype[1] == "mm") {
       # multi-membership grouping term
-      stopifnot(!nzchar(id_ranef$by[1]))
       gs <- id_ranef$gcall[[1]]$groups
       ngs <- length(gs)
       weights <- id_ranef$gcall[[1]]$weights
@@ -256,8 +257,15 @@ data_gr_local <- function(bterms, data, ranef) {
         weights <- matrix(1 / ngs, nrow = nrow(data), ncol = ngs)
       }
       for (i in seq_along(gs)) {
-        J <- as.array(match(get(gs[i], data), levels))
-        out[[paste0("J_", idresp, "_", i)]] <- J
+        gdata <- get(gs[i], data)
+        J <- match(gdata, levels)
+        if (anyNA(J)) {
+          # occurs for new levels only
+          new_gdata <- gdata[!gdata %in% levels]
+          new_levels <- unique(new_gdata)
+          J[is.na(J)] <- match(new_gdata, new_levels) + length(levels)
+        }
+        out[[paste0("J_", idresp, "_", i)]] <- as.array(J)
         out[[paste0("W_", idresp, "_", i)]] <- as.array(weights[, i])
       }
     } else {
@@ -572,6 +580,8 @@ data_gp <- function(bterms, data, internal = FALSE, basis = NULL, ...) {
   if (internal) {
     # required for centering of approximate GPs with new data
     out[[paste0("cmeans", sfx)]] <- cmeans
+    # required to compute inverse-gamma priors for length-scales
+    out[[paste0("Xgp_prior", sfx)]] <- Xgp
   }
   if (!isNA(k)) {
     # basis function approach requires centered variables
@@ -821,30 +831,41 @@ data_cnl <- function(bterms, data) {
 }
 
 # data for special priors such as horseshoe and lasso
-data_prior <- function(bterms, data, prior) {
+data_prior <- function(bterms, data, prior, sdata = NULL) {
   out <- list()
   px <- check_prefix(bterms)
   p <- usc(combine_prefix(px))
-  prefix <- combine_prefix(px, keep_mu = TRUE)
-  special <- attr(prior, "special")[[prefix]]
-  if (!is.null(special[["hs_df"]])) {
+  special <- get_special_prior(prior, px)
+  if (!is.null(special$horseshoe)) {
     # data for the horseshoe prior
-    hs_obj_names <- paste0("hs_", 
-      c("df", "df_global", "df_slab", "scale_global", "scale_slab")
-    )
-    hs_data <- special[hs_obj_names]
-    if (is.null(special[["hs_par_ratio"]])) {
-      hs_data$hs_scale_global <- special$hs_scale_global
-    } else {
-      hs_data$hs_scale_global <- special$hs_par_ratio / sqrt(nrow(data))
+    hs_names <- c("df", "df_global", "df_slab", "scale_global", "scale_slab")
+    hs_data <- special$horseshoe[hs_names]
+    if (!is.null(special$horseshoe$par_ratio)) {
+      hs_data$scale_global <- special$horseshoe$par_ratio / sqrt(nrow(data))
     }
-    names(hs_data) <- paste0(hs_obj_names, p) 
+    names(hs_data) <- paste0("hs_", hs_names, p) 
     out <- c(out, hs_data)
   }
-  if (!is.null(special[["lasso_df"]])) {
-    lasso_obj_names <- paste0("lasso_", c("df", "scale"))
-    lasso_data <- special[lasso_obj_names]
-    names(lasso_data) <- paste0(lasso_obj_names, p) 
+  if (!is.null(special$R2D2)) {
+    # data for the R2D2 prior
+    R2D2_names <- c("mean_R2", "prec_R2", "cons_D2")
+    R2D2_data <- special$R2D2[R2D2_names]
+    # number of coefficients minus the intercept
+    K <- sdata[[paste0("K", p)]] - ifelse(stan_center_X(bterms), 1, 0)
+    if (length(R2D2_data$cons_D2) == 1L) {
+      R2D2_data$cons_D2 <- rep(R2D2_data$cons_D2, K)
+    }
+    if (length(R2D2_data$cons_D2) != K) {
+      stop2("Argument 'cons_D2' of the R2D2 prior must be of length 1 or ", K)
+    }
+    R2D2_data$cons_D2 <- as.array(R2D2_data$cons_D2)
+    names(R2D2_data) <- paste0("R2D2_", R2D2_names, p) 
+    out <- c(out, R2D2_data)
+  }
+  if (!is.null(special$lasso)) {
+    lasso_names <- c("df", "scale")
+    lasso_data <- special$lasso[lasso_names]
+    names(lasso_data) <- paste0("lasso_", lasso_names, p) 
     out <- c(out, lasso_data)
   }
   out
@@ -898,9 +919,12 @@ smoothCon <- function(object, data, ...) {
   data <- rm_attr(data, "terms")
   vars <- setdiff(c(object$term, object$by), "NA")
   for (v in vars) {
-    # allow factor-like variables #562
     if (is_like_factor(data[[v]])) {
+      # allow factor-like variables #562
       data[[v]] <- as.factor(data[[v]])
+    } else if (inherits(data[[v]], "difftime")) {
+      # mgcv cannot handle 'difftime' variables
+      data[[v]] <- as.numeric(data[[v]])
     }
   }
   mgcv::smoothCon(object, data = data, ...)
