@@ -77,7 +77,7 @@ rstudent_t <- function(n, df, mu = 0, sigma = 1) {
 #'   should be performed. Defaults to \code{FALSE} to improve
 #'   efficiency.
 #'   
-#' @details See the Stan user's manual \url{http://mc-stan.org/documentation/}
+#' @details See the Stan user's manual \url{https://mc-stan.org/documentation/}
 #' for details on the parameterization
 #'   
 #' @export
@@ -143,7 +143,7 @@ rmulti_normal <- function(n, mu, Sigma, check = FALSE) {
 #'   should be performed. Defaults to \code{FALSE} to improve
 #'   efficiency.
 #'   
-#' @details See the Stan user's manual \url{http://mc-stan.org/documentation/}
+#' @details See the Stan user's manual \url{https://mc-stan.org/documentation/}
 #'   for details on the parameterization
 #'   
 #' @export
@@ -1640,7 +1640,7 @@ pcox <- function(q, mu, bhaz, cbhaz, lower.tail = TRUE, log.p = FALSE) {
 #' @name ZeroInflated
 #' 
 #' @inheritParams StudentT
-#' @param zi zero-inflation propability
+#' @param zi zero-inflation probability
 #' @param mu,lambda location parameter
 #' @param shape,shape1,shape2 shape parameter
 #' @param size number of trials
@@ -1802,7 +1802,7 @@ pzero_inflated_asym_laplace <- function(q, mu, sigma, quantile, zi,
 #' @name Hurdle
 #' 
 #' @inheritParams StudentT
-#' @param hu hurdle propability
+#' @param hu hurdle probability
 #' @param mu,lambda location parameter
 #' @param shape shape parameter
 #' @param sigma,scale scale parameter
@@ -1966,7 +1966,7 @@ dcategorical <- function(x, eta, log = FALSE) {
   } else {
     out <- softmax(eta)
   }
-  out[, x]
+  out[, x, drop = FALSE]
 }
 
 # CDF of the categorical distribution with the softmax transform
@@ -1975,7 +1975,7 @@ dcategorical <- function(x, eta, log = FALSE) {
 # @param log.p return values on the log scale?
 pcategorical <- function(q, eta, log.p = FALSE) {
   p <- dcategorical(seq_len(max(q)), eta = eta)
-  out <- do_call(cbind, lapply(q, function(j) rowSums(as.matrix(p[, 1:j]))))
+  out <- cblapply(q, function(j) rowSums(p[, 1:j, drop = FALSE]))
   if (log.p) {
     out <- log(out)
   }
@@ -2093,14 +2093,14 @@ dacat <- function(x, eta, thres, disc = 1, link = "logit") {
 # @param disc samples of the discrimination parameter
 # @param family a character string naming the family
 # @param link a character string naming the link
-# @return a matrix of probabilites P(x <= q)
+# @return a matrix of probabilities P(x <= q)
 pordinal <- function(q, eta, thres, disc = 1, family = NULL, link = "logit") {
   family <- as_one_character(family)
   link <- as_one_character(link)
   args <- nlist(x = seq_len(max(q)), eta, thres, disc, link)
   p <- do_call(paste0("d", family), args)
   .fun <- function(j) rowSums(as.matrix(p[, 1:j, drop = FALSE]))
-  do_call(cbind, lapply(q, .fun))
+  cblapply(q, .fun)
 }
 
 # helper functions to shift arbitrary distributions
