@@ -27,6 +27,8 @@
 #'   estimates are treated as problematic. Defaults to \code{0.7}. 
 #'   Only used if argument \code{reloo} is \code{TRUE}.
 #'   See \code{\link[loo:pareto-k-diagnostic]{pareto_k_ids}} for more details.
+#' @param save_psis Should the \code{"psis"} object created internally be saved
+#'   in the returned object? For more details see \code{\link[loo:loo]{loo}}.
 #' @param moment_match_args Optional \code{list} of additional arguments passed to
 #'   \code{\link{loo_moment_match}}.
 #' @param reloo_args Optional \code{list} of additional arguments passed to
@@ -78,14 +80,14 @@
 #' @export
 loo.brmsfit <-  function(x, ..., compare = TRUE, resp = NULL,
                          pointwise = FALSE, moment_match = FALSE,
-                         reloo = FALSE, k_threshold = 0.7,
+                         reloo = FALSE, k_threshold = 0.7, save_psis = FALSE,
                          moment_match_args = list(), reloo_args = list(), 
                          model_names = NULL) {
   args <- split_dots(x, ..., model_names = model_names)
   c(args) <- nlist(
     criterion = "loo", pointwise, compare, 
-    resp, k_threshold, moment_match, reloo, 
-    moment_match_args, reloo_args
+    resp, k_threshold, save_psis, moment_match, 
+    reloo, moment_match_args, reloo_args
   )
   do_call(compute_loolist, args)
 }
@@ -93,7 +95,7 @@ loo.brmsfit <-  function(x, ..., compare = TRUE, resp = NULL,
 #' @export
 LOO.brmsfit <- function(x, ..., compare = TRUE, resp = NULL,
                         pointwise = FALSE, moment_match = FALSE,
-                        reloo = FALSE, k_threshold = 0.7,
+                        reloo = FALSE, k_threshold = 0.7, save_psis = FALSE,
                         moment_match_args = list(), reloo_args = list(), 
                         model_names = NULL) {
   cl <- match.call()
@@ -253,10 +255,11 @@ loo_criteria <- function() {
 # compute 'loo' criterion using the 'loo' package
 .loo <- function(x, pointwise, k_threshold, moment_match, reloo, 
                  moment_match_args, reloo_args, newdata, 
-                 resp, model_name, ...) {
+                 resp, model_name, save_psis, ...) {
   loo_args <- prepare_loo_args(
     x, newdata = newdata, resp = resp, 
-    pointwise = pointwise, ...
+    pointwise = pointwise, save_psis = save_psis, 
+    ...
   )
   out <- SW(do_call("loo", loo_args, pkg = "loo"))
   if (moment_match) {
@@ -476,8 +479,8 @@ add_criterion.brmsfit <- function(x, criterion, model_name = NULL,
     warning2("Criterion 'R2' is deprecated. Please use 'bayes_R2' instead.")
     criterion[criterion == "R2"] <- "bayes_R2"
   }
-  loo_options <- c("loo", "waic", "kfold", "loo_subsample", "loo_R2")
-  options <- c(loo_options, "bayes_R2", "marglik")
+  loo_options <- c("loo", "waic", "kfold", "loo_subsample")
+  options <- c(loo_options, "bayes_R2", "loo_R2", "marglik")
   if (!length(criterion) || !all(criterion %in% options)) {
     stop2("Argument 'criterion' should be a subset of ",
           collapse_comma(options))
@@ -505,6 +508,10 @@ add_criterion.brmsfit <- function(x, criterion, model_name = NULL,
     args$summary <- FALSE
     x$criteria$bayes_R2 <- do_call(bayes_R2, args)
   }
+  if ("loo_R2" %in% criterion) {
+    args$summary <- FALSE
+    x$criteria$loo_R2 <- do_call(loo_R2, args)
+  }
   if ("marglik" %in% criterion) {
     x$criteria$marglik <- do_call(bridge_sampler, args)
   }
