@@ -23,21 +23,21 @@
 #'   to work correctly.
 #' @return An updated object of class \code{loo}.
 #' 
-#' @details The moment matching algorithm requires samples 
-#'   of all variables defined in Stan's \code{parameters} block
-#'   to be saved. Otherwise \code{loo_moment_match} cannot be computed.
-#'   Thus, please set \code{save_all_pars = TRUE} in the call to \code{brm},
-#'   if you are planning to apply \code{loo_moment_match} to your models.
+#' @details The moment matching algorithm requires samples of all variables
+#'   defined in Stan's \code{parameters} block to be saved. Otherwise
+#'   \code{loo_moment_match} cannot be computed. Thus, please set
+#'   \code{save_pars = save_pars(all = TRUE)} in the call to \code{brm}, if you
+#'   are planning to apply \code{loo_moment_match} to your models.
 #'   
 #' @references 
-#'   Paananen, T., Piironen, J., Buerkner, P.-C., Vehtari, A. (2020). 
-#'   Implicitly Adaptive Importance Sampling. preprint arXiv:1906.08850
+#'   Paananen, T., Piironen, J., Buerkner, P.-C., Vehtari, A. (2021). 
+#'   Implicitly Adaptive Importance Sampling. Statistics and Computing.
 #' 
 #' @examples 
 #' \dontrun{
 #' fit1 <- brm(count ~ zAge + zBase * Trt + (1|patient),
 #'             data = epilepsy, family = poisson(),
-#'             save_all_pars = TRUE)
+#'             save_pars = save_pars(all = TRUE))
 #'             
 #' # throws warning about some pareto k estimates being too high
 #' (loo1 <- loo(fit1))
@@ -83,7 +83,7 @@ loo_moment_match.brmsfit <- function(x, loo, k_threshold = 0.7, newdata = NULL,
   if (is(out, "try-error")) {
     stop2(
       "Moment matching failed. Perhaps you did not set ", 
-      "'save_all_pars' to TRUE when fitting your model?"
+      "'save_pars = save_pars(all = TRUE)' when fitting your model?"
     )
   }
   out
@@ -111,6 +111,8 @@ loo_moment_match.brmsfit <- function(x, loo, k_threshold = 0.7, newdata = NULL,
 .update_pars <- function(x, upars, ...) {
   # list with one element per posterior draw
   pars <- apply(upars, 1, .constrain_pars, x = x)
+  # select required parameters only
+  pars <- lapply(pars, "[", x$fit@sim$pars_oi_old)
   # transform samples
   nsamples <- length(pars)
   pars <- unlist(pars)
@@ -120,7 +122,10 @@ loo_moment_match.brmsfit <- function(x, loo, k_threshold = 0.7, newdata = NULL,
   pars <- rbind(pars, rep(0, nsamples))
   # bring samples into the right structure
   new_samples <- named_list(x$fit@sim$fnames_oi_old, list(numeric(nsamples)))
-  stopifnot(length(new_samples) == nrow(pars))
+  if (length(new_samples) != nrow(pars)) {
+    stop2("Updating parameters in `loo_moment_match.brmsfit' failed. ",
+          "Please report a bug at https://github.com/paul-buerkner/brms.")
+  }
   for (i in seq_len(npars)) {
     new_samples[[i]] <- pars[i, ]
   }
