@@ -103,7 +103,7 @@
 #'   represents the expected response value when all predictors 
 #'   are at their means. To treat the intercept as an ordinary 
 #'   population-level effect and avoid the centering parameterization, 
-#'   use \code{0 + intercept} on the right-hand side of the model formula.
+#'   use \code{0 + Intercept} on the right-hand side of the model formula.
 #'   
 #'   A special shrinkage prior to be applied on population-level effects is the
 #'   (regularized) horseshoe prior and related priors. See
@@ -145,8 +145,8 @@
 #'   These parameters are restricted to be non-negative and, by default, 
 #'   have a half student-t prior with 3 degrees of freedom and a 
 #'   scale parameter that depends on the standard deviation of the response 
-#'   after applying the link function. Minimally, the scale parameter is 10. 
-#'   This prior is used (a) to be only very weakly informative in order to influence
+#'   after applying the link function. Minimally, the scale parameter is 2.5. 
+#'   This prior is used (a) to be only weakly informative in order to influence
 #'   results as few as possible, while (b) providing at least some regularization
 #'   to considerably improve convergence and sampling efficiency.
 #'   To define a prior distribution only for standard deviations 
@@ -284,9 +284,6 @@
 #'   Fixing parameters to constants is possible by using the \code{constant}
 #'   function, for example, \code{constant(1)} to fix a parameter to 1.
 #'   Broadcasting to vectors and matrices is done automatically.
-#'   A limitation of the current implementation is that the same parameter
-#'   vector cannot contain estimated and fixed values at the same time,
-#'   but this will be possible in the future.
 #' 
 #'   Often, it may not be immediately clear, which parameters are present in the
 #'   model. To get a full list of parameters and parameter classes for which
@@ -480,7 +477,7 @@ prior_string <- function(prior, ...) {
 #' 
 #' @export
 get_prior <- function(formula, data, family = gaussian(), autocor = NULL, 
-                      knots = NULL, sparse = NULL, ...) {
+                      data2 = NULL, knots = NULL, sparse = NULL, ...) {
   if (is.brmsfit(formula)) {
     stop2("Use 'prior_summary' to extract priors from 'brmsfit' objects.")
   }
@@ -489,7 +486,14 @@ get_prior <- function(formula, data, family = gaussian(), autocor = NULL,
     autocor = autocor, sparse = sparse
   )
   bterms <- brmsterms(formula)
-  data <- validate_data(data, bterms = bterms, knots = knots)
+  data2 <- validate_data2(
+    data2, bterms = bterms, 
+    get_data2_autocor(formula)
+  )
+  data <- validate_data(
+    data, bterms = bterms, 
+    data2 = data2, knots = knots
+  )
   .get_prior(bterms, data, ...)
 }
   
@@ -971,7 +975,7 @@ prior_ac <- function(bterms, def_scale_prior, ...) {
   if (has_ac_class(acef, "cosy")) {
     prior <- prior + brmsprior(class = "cosy", ls = px)
   }
-  if (has_cor_latent_residuals(bterms)) {
+  if (has_ac_latent_residuals(bterms)) {
     prior <- prior + 
       brmsprior(def_scale_prior, class = "sderr", ls = px)
   }
@@ -1101,14 +1105,47 @@ def_scale_prior.brmsterms <- function(x, data, center = TRUE, df = 3,
   paste0("student_t(", sargs(df, location, scale), ")")
 }
 
-# validate priors supplied by the user
-# @param ... passed to 'validate_prior_special*
-# @return a 'brmsprior' object
-validate_prior <- function(prior, bterms, data, sample_prior = "no", ...) {
+#' Validate Prior for \pkg{brms} Models
+#' 
+#' Validate priors supplied by the user. Return a complete
+#' set of priors for the given model, including default priors.
+#' 
+#' @inheritParams get_prior
+#' @inheritParams brm
+#' 
+#' @return An object of class \code{brmsprior}.
+#' 
+#' @seealso \code{\link{get_prior}}, \code{\link{set_prior}}.
+#' 
+#' @examples 
+#' prior1 <- prior(normal(0,10), class = b) + 
+#'   prior(cauchy(0,2), class = sd)
+#' validate_prior(prior1, count ~ zAge + zBase * Trt + (1|patient),
+#'                data = epilepsy, family = poisson())
+#' 
+#' @export
+validate_prior <- function(prior, formula, data, family = gaussian(),
+                           sample_prior = "no", data2 = NULL, knots = NULL, 
+                           ...) {
+  formula <- validate_formula(formula, data = data, family = family)
+  bterms <- brmsterms(formula)
+  data2 <- validate_data2(data2, bterms = bterms)
+  data <- validate_data(
+    data, bterms = bterms, 
+    data2 = data2, knots = knots
+  )
+  .validate_prior(
+    prior, bterms = bterms, data = data,
+    sample_prior = sample_prior, ...
+  )
+}  
+
+# internal work function of 'validate_prior'
+.validate_prior <- function(prior, bterms, data, sample_prior, ...) {
   sample_prior <- validate_sample_prior(sample_prior)
   all_priors <- .get_prior(bterms, data, internal = TRUE)
   if (is.null(prior)) {
-    prior <- all_priors  
+    prior <- all_priors
   } else if (!is.brmsprior(prior)) {
     stop2("Argument 'prior' must be a 'brmsprior' object.")
   }
@@ -1400,7 +1437,7 @@ validate_prior_special.btl <- function(x, prior, data,
     }
   }
   # prepare special priors such as horseshoe or lasso
-  prior_special <- list()
+  special <- list()
   b_index <- which(find_rows(prior, class = "b", coef = "", ls = px))
   stopifnot(length(b_index) <= 1L)
   if (length(b_index)) {
@@ -1427,37 +1464,27 @@ validate_prior_special.btl <- function(x, prior, data,
         )
       }
       if (is_special_prior(b_prior, "horseshoe")) {
-        hs <- eval2(b_prior)
-        hs_obj_names <- c(
-          "df", "df_global", "df_slab", "scale_global", 
-          "scale_slab", "par_ratio", "autoscale"
-        )
-        hs_att <- attributes(hs)[hs_obj_names]
-        names(hs_att) <- paste0("hs_", hs_obj_names)
-        prior_special <- c(prior_special, hs_att)
-        prior_special$hs_autoscale <- 
-          isTRUE(prior_special$hs_autoscale) && allow_autoscale
+        special$horseshoe <- attributes(eval2(b_prior))
+        special$horseshoe$autoscale <- 
+          isTRUE(special$horseshoe$autoscale) && allow_autoscale
+      } else if (is_special_prior(b_prior, "R2D2")) {
+        special$R2D2 <- attributes(eval2(b_prior))
+        special$R2D2$autoscale <- 
+          isTRUE(special$R2D2$autoscale) && allow_autoscale
       } else if (is_special_prior(b_prior, "lasso")) {
-        lasso <- eval2(b_prior)
         # the parameterization via double_exponential appears to be more
         # efficient than an indirect parameterization via normal and 
         # exponential distributions; tested on 2017-06-09
         p <- usc(combine_prefix(px))
-        lasso_scale <- paste0(
-          "lasso_scale", p, " * lasso_inv_lambda", p
-        )
-        lasso_prior <- paste0(
-          "double_exponential(0, ", lasso_scale, ")"
-        )
+        lasso_scale <- paste0("lasso_scale", p, " * lasso_inv_lambda", p)
+        lasso_prior <- paste0("double_exponential(0, ", lasso_scale, ")")
         prior$prior[b_index] <- lasso_prior
-        lasso_att <- attributes(lasso)
-        prior_special$lasso_df <- lasso_att[["df"]]
-        prior_special$lasso_scale <- lasso_att[["scale"]]
+        special$lasso <- attributes(eval2(b_prior))
       }
     }
   }
   prefix <- combine_prefix(px, keep_mu = TRUE)
-  attributes(prior)$special[[prefix]] <- prior_special
+  attributes(prior)$special[[prefix]] <- special
   prior
 }
 
@@ -1896,6 +1923,57 @@ horseshoe <- function(df = 1, scale_global = 1, df_global = 1,
   out
 }
 
+#' R2-D2 Priors in \pkg{brms}
+#' 
+#' Function used to set up R2D2 priors for population-level effects in
+#' \pkg{brms}. The function does not evaluate its arguments -- it exists purely
+#' to help set up the model.
+#' 
+#' @param mean_R2 mean of the Beta prior on the coefficient of determination R^2.
+#' @param prec_R2 precision of the Beta prior on the coefficient of determination R^2.
+#' @param cons_D2 concentration vector of the Dirichlet prior on the variance
+#'   decomposition parameters.
+#' @param autoscale Logical; indicating whether the horseshoe
+#'   prior should be scaled using the residual standard deviation
+#'   \code{sigma} if possible and sensible (defaults to \code{TRUE}).
+#'   Autoscaling is not applied for distributional parameters or 
+#'   when the model does not contain the parameter \code{sigma}.
+#'   
+#' @references
+#' Zhang, Y. D., Naughton, B. P., Bondell, H. D., & Reich, B. J. (2020). 
+#'   Bayesian regression using a prior on the model fit: The R2-D2 shrinkage 
+#'   prior. Journal of the American Statistical Association.
+#'   \url{https://arxiv.org/pdf/1609.00046.pdf}
+#'   
+#' @seealso \code{\link{set_prior}}
+#'   
+#' @examples 
+#' set_prior(R2D2(mean_R2 = 0.8, prec_R2 = 10))
+#' 
+#' @export
+R2D2 <- function(mean_R2 = 0.5, prec_R2 = 2, cons_D2 = 1, autoscale = TRUE) {
+  out <- deparse(match.call(), width.cutoff = 500L)
+  mean_R2 <- as_one_numeric(mean_R2)
+  prec_R2 <- as_one_numeric(prec_R2)
+  cons_D2 <- as.numeric(cons_D2)
+  if (!(mean_R2 > 0 && mean_R2 < 1)) {
+    stop2("Invalid R2D2 prior: Mean of the R2 prior ",
+          "must be a single number in (0, 1).")
+  }
+  if (prec_R2 <= 0) {
+    stop2("Invalid R2D2 prior: Precision of the R2 prior ",
+          "must be a single positive number.")
+  }
+  if (any(cons_D2 <= 0)) {
+    stop2("Invalid R2D2 prior: Concentration of the D2 prior ",
+          "must be a vector of positive numbers.")
+  }
+  autoscale <- as_one_logical(autoscale)
+  att <- nlist(mean_R2, prec_R2, cons_D2, autoscale)
+  attributes(out)[names(att)] <- att
+  out
+}
+
 #' Set up a lasso prior in \pkg{brms}
 #' 
 #' Function used to set up a lasso prior for population-level effects 
@@ -1962,12 +2040,24 @@ lasso <- function(df = 1, scale = 1) {
 is_special_prior <- function(prior, target = NULL) {
   stopifnot(is.character(prior))
   if (is.null(target)) {
-    target <- c("horseshoe", "lasso") 
+    target <- c("horseshoe", "R2D2", "lasso") 
   }
   regex <- paste0("^", regex_or(target), "\\(")
   grepl(regex, prior)
 }
 
+# extract special prior information
+# @param prior a brmsprior object
+# @param px object from which the prefix can be extract
+get_special_prior <- function(prior, px = NULL) {
+  out <- attr(prior, "special")
+  if (!is.null(px)) {
+    prefix <- combine_prefix(px, keep_mu = TRUE)
+    out <- out[[prefix]]
+  }
+  out
+}
+
 # check if parameters should be samples only from the prior
 is_prior_only <- function(prior) {
   is_equal(get_sample_prior(prior), "only")
