@@ -2,8 +2,9 @@
 # of Stan code snippets to be pasted together later on
 
 # Stan code for the response variables
-stan_response <- function(bterms, data) {
+stan_response <- function(bterms, data, normalize) {
   stopifnot(is.brmsterms(bterms))
+  lpdf <- stan_lpdf_name(normalize)
   family <- bterms$family
   rtype <- str_if(use_int(family), "int", "real")
   multicol <- has_multicol(family)
@@ -196,7 +197,7 @@ stan_response <- function(bterms, data) {
         "  vector{Ybounds}[N{resp}] Yl{resp};  // latent variable\n"
       )
       str_add(out$prior) <- glue(
-        "  target += normal_lpdf(Y{resp}[Jme{resp}]",
+        "  target += normal_{lpdf}(Y{resp}[Jme{resp}]",
         " | Yl{resp}[Jme{resp}], noise{resp}[Jme{resp}]);\n"
       )
       str_add(out$pll_args) <- glue(", vector Yl{resp}")
@@ -229,7 +230,7 @@ stan_response <- function(bterms, data) {
 # intercepts in ordinal models require special treatment
 # and must be present even when using non-linear predictors
 # thus the relevant Stan code cannot be part of 'stan_fe'
-stan_thres <- function(bterms, data, prior, ...) {
+stan_thres <- function(bterms, data, prior, normalize, ...) {
   stopifnot(is.btl(bterms) || is.btnl(bterms))
   out <- list()
   if (!is_ordinal(bterms)) {
@@ -270,12 +271,12 @@ stan_thres <- function(bterms, data, prior, ...) {
           prior, class = "Intercept", group = groups[i], 
           type = "real", prefix = "first_",
           suffix = glue("{p}{gr[i]}"), px = px, 
-          comment = "first threshold"
+          comment = "first threshold", normalize = normalize
         )
         str_add_list(out) <- stan_prior(
           prior, class = "delta", group = groups[i], 
           type = glue("real{bound}"), px = px, suffix = gr[i], 
-          comment = "distance between thresholds"
+          comment = "distance between thresholds", normalize = normalize
         )
       }
       str_add(out$tpar_def) <- 
@@ -298,7 +299,8 @@ stan_thres <- function(bterms, data, prior, ...) {
           coef = get_thres(bterms, group = groups[i]), 
           type = glue("{type}[nthres{resp}{grb[i]}]"),
           coef_type = coef_type, px = px, suffix = glue("{p}{gr[i]}"),
-          comment = "temporary thresholds for centered predictors"
+          comment = "temporary thresholds for centered predictors",
+          normalize = normalize
         )
       }
     }
@@ -346,12 +348,13 @@ stan_thres <- function(bterms, data, prior, ...) {
 }
 
 # Stan code for the baseline functions of the Cox model
-stan_bhaz <- function(bterms, prior, threads, ...) {
+stan_bhaz <- function(bterms, prior, threads, normalize, ...) {
   stopifnot(is.btl(bterms) || is.btnl(bterms))
   out <- list()
   if (!is_cox(bterms$family)) {
     return(out)
   }
+  lpdf <- stan_lpdf_name(normalize)
   px <- check_prefix(bterms)
   p <- usc(combine_prefix(px))
   resp <- usc(px$resp)
@@ -370,7 +373,7 @@ stan_bhaz <- function(bterms, prior, threads, ...) {
     "  simplex[Kbhaz{resp}] sbhaz{resp};  // baseline coefficients\n"
   )
   str_add(out$prior) <- glue(
-    "  target += dirichlet_lpdf(sbhaz{resp} | con_sbhaz{resp});\n"
+    "  target += dirichlet_{lpdf}(sbhaz{resp} | con_sbhaz{resp});\n"
   )
   str_add(out$model_def) <- glue(
     "  // compute values of baseline function\n",
@@ -385,11 +388,12 @@ stan_bhaz <- function(bterms, prior, threads, ...) {
 }
 
 # Stan code specific to mixture families
-stan_mixture <- function(bterms, data, prior, threads, ...) {
+stan_mixture <- function(bterms, data, prior, threads, normalize, ...) {
   out <- list()
   if (!is.mixfamily(bterms$family)) {
     return(out)
   }
+  lpdf <- stan_lpdf_name(normalize)
   px <- check_prefix(bterms)
   p <- usc(combine_prefix(px))
   n <- stan_nn(threads)
@@ -440,7 +444,7 @@ stan_mixture <- function(bterms, data, prior, threads, ...) {
       "  simplex[{nmix}] theta{p};  // mixing proportions\n"
     )
     str_add(out$prior) <- glue(
-      "  target += dirichlet_lpdf(theta{p} | con_theta{p});\n"                
+      "  target += dirichlet_{lpdf}(theta{p} | con_theta{p});\n"                
     )
     # separate definition from computation to support fixed parameters
     str_add(out$tpar_def) <- "  // mixing proportions\n"
@@ -477,7 +481,8 @@ stan_mixture <- function(bterms, data, prior, threads, ...) {
         type = glue("{type}[nthres{p}{grb[i]}]"), 
         coef_type = coef_type, px = px, 
         prefix = "fixed_", suffix = glue("{p}{gr[i]}"),
-        comment = "thresholds fixed over mixture components"
+        comment = "thresholds fixed over mixture components",
+        normalize = normalize
       )
     }
   }
@@ -512,50 +517,95 @@ stan_ordinal_lpmf <- function(family, link) {
   )
   # define the function body
   if (family == "cumulative") {
-    str_add(out) <- glue(
-      "     int nthres = num_elements(thres);\n",
-      "     real p;\n",
-      "     if (y == 1) {{\n",
-      "       p = {ilink}({th(1)});\n",
-      "     }} else if (y == nthres + 1) {{\n",
-      "       p = 1 - {ilink}({th('nthres')});\n",
-      "     }} else {{\n",
-      "       p = {ilink}({th('y')}) -\n",
-      "           {ilink}({th('y - 1')});\n",
-      "     }}\n",
-      "     return log(p);\n",
-      "   }}\n"
-    )
+    if (ilink == "inv_logit") {
+      str_add(out) <- glue(
+        "     int nthres = num_elements(thres);\n",
+        "     if (y == 1) {{\n",
+        "       return log_inv_logit({th(1)});\n",
+        "     }} else if (y == nthres + 1) {{\n",
+        "       return log1m_inv_logit({th('nthres')});\n",
+        "     }} else {{\n",
+        # TODO: replace with log_inv_logit_diff once rstan >= 2.25
+        "       return log_diff_exp(\n",
+        "         log_inv_logit({th('y')}), \n",
+        "         log_inv_logit({th('y - 1')})\n",
+        "       );\n",
+        "     }}\n",
+        "   }}\n"
+      )
+    } else {
+      str_add(out) <- glue(
+        "     int nthres = num_elements(thres);\n",
+        "     real p;\n",
+        "     if (y == 1) {{\n",
+        "       p = {ilink}({th(1)});\n",
+        "     }} else if (y == nthres + 1) {{\n",
+        "       p = 1 - {ilink}({th('nthres')});\n",
+        "     }} else {{\n",
+        "       p = {ilink}({th('y')}) -\n",
+        "           {ilink}({th('y - 1')});\n",
+        "     }}\n",
+        "     return log(p);\n",
+        "   }}\n"
+      )
+    }
   } else if (family %in% c("sratio", "cratio")) {
-    sc <- str_if(family == "sratio", "1 - ")
+    if (ilink == "inv_cloglog") {
+      qk <- str_if(
+        family == "sratio", 
+        "-exp({th('k')})",
+        "log1m_exp(-exp({th('k')}))"
+      )
+    } else if (ilink == "inv_logit") {
+      qk <- str_if(
+        family == "sratio", 
+        "log1m_inv_logit({th('k')})",
+        "log_inv_logit({th('k')})"
+      )
+    } else if (ilink == "Phi") {
+      # TODO: replace with more stable std_normal_lcdf once rstan >= 2.25
+      qk <- str_if(
+        family == "sratio", 
+        "normal_lccdf({th('k')}|0,1)",
+        "normal_lcdf({th('k')}|0,1)"
+      )
+    } else if (ilink == "Phi_approx") {
+      qk <- str_if(
+        family == "sratio",
+        "log1m_inv_logit(0.07056 * pow({th('k')}, 3.0) + 1.5976 * {th('k')})",
+        "log_inv_logit(0.07056 * pow({th('k')}, 3.0) + 1.5976 * {th('k')})"
+      )
+    } else if (ilink == "inv_cauchit") {
+      qk <- str_if(
+        family == "sratio",
+        "cauchy_lccdf({th('k')}|0,1)",
+        "cauchy_lcdf({th('k')}|0,1)"
+      )
+    }
+    qk <- glue(qk)
     str_add(out) <- glue(
       "     int nthres = num_elements(thres);\n",
       "     vector[nthres + 1] p;\n",
       "     vector[nthres] q;\n",
       "     int k = 1;\n",
       "     while (k <= min(y, nthres)) {{\n",
-      "       q[k] = {sc}{ilink}({th('k')});\n",
-      "       p[k] = 1 - q[k];\n",
-      "       for (kk in 1:(k - 1)) p[k] = p[k] * q[kk];\n", 
+      "       q[k] = {qk};\n",
+      "       p[k] = log1m_exp(q[k]);\n",
+      "       for (kk in 1:(k - 1)) p[k] = p[k] + q[kk];\n", 
       "       k += 1;\n",
       "     }}\n",
       "     if (y == nthres + 1) {{\n",
-      "       p[nthres + 1] = prod(q);\n",
+      "       p[nthres + 1] = sum(q);\n",
       "     }}\n",
-      "     return log(p[y]);\n",
+      "     return p[y];\n",
       "   }}\n"
     )
   } else if (family == "acat") {
     if (ilink == "inv_logit") {
       str_add(out) <- glue(
         "     int nthres = num_elements(thres);\n",
-        "     vector[nthres + 1] p;\n",
-        "     p[1] = 0.0;\n",
-        "     for (k in 1:(nthres)) {{\n",
-        "       p[k + 1] = p[k] + {th('k')};\n",
-        "     }}\n",
-        "     p = exp(p);\n",
-        "     return log(p[y] / sum(p));\n",
+        "     vector[nthres + 1] p = append_row(0, cumulative_sum(disc * (mu - thres)));\n",
+        "     return p[y] - log_sum_exp(p);\n",
         "   }}\n"
       )
     } else {
@@ -575,7 +625,7 @@ stan_ordinal_lpmf <- function(family, link) {
       )
     }
   }
-  # lpdf function for multiple merged thresholds
+  # lpmf function for multiple merged thresholds
   str_add(out) <- glue(
     "  /* {family}-{link} log-PDF for a single response and merged thresholds\n",
     "   * Args:\n",
