@@ -11,7 +11,7 @@
 #'   Only required if \code{block = 'data'} and ignored otherwise.
 #' @param name Optional character string providing the desired variable 
 #'  name of the object in \code{x}. If \code{NULL} (the default)
-#'  the variable name is directly infered from \code{x}.
+#'  the variable name is directly inferred from \code{x}.
 #' @param scode Line of Stan code to define the variable
 #'  in Stan language. If \code{block = 'data'}, the
 #'  Stan code is inferred based on the class of \code{x} by default.
@@ -19,10 +19,15 @@
 #'  which the variable should be defined. Can be \code{'data'},
 #'  \code{'tdata'} (transformed data), \code{'parameters'},
 #'  \code{'tparameters'} (transformed parameters), \code{'model'},
+#'  \code{'likelihood'} (part of the model block where the likelihood is given),
 #'  \code{'genquant'} (generated quantities) or \code{'functions'}.
 #' @param position Name of the position within the block where the
 #'  Stan code should be placed. Currently allowed are \code{'start'}
 #'  (the default) and \code{'end'} of the block.
+#' @param pll_args Optional Stan code to be put into the header
+#'  of \code{partial_log_lik} functions. This ensures that the variables
+#'  specified in \code{scode} can be used in the likelihood even when
+#'  within-chain parallelization is activated via \code{\link{threading}}.
 #'  
 #' @return An object of class \code{stanvars}.
 #' 
@@ -46,13 +51,21 @@
 #'                     block = "parameters")
 #' make_stancode(count ~ Trt + zBase, epilepsy,
 #'               prior = bprior, stanvars = stanvars)
+#'               
+#' # ensure that 'tau' is passed to the likelihood of a threaded model
+#' # not necessary for this example but may be necessary in other cases
+#' stanvars <- stanvar(scode = "real<lower=0> tau;", 
+#'                     block = "parameters", pll_args = "real tau")
+#' make_stancode(count ~ Trt + zBase, epilepsy,
+#'               stanvars = stanvars, threads = threading(2))
 #' 
 #' @export
 stanvar <- function(x = NULL, name = NULL, scode = NULL,
-                    block = "data", position = "start") {
+                    block = "data", position = "start",
+                    pll_args = NULL) {
   vblocks <- c(
     "data", "tdata", "parameters", "tparameters", 
-    "model", "genquant", "functions"
+    "model", "genquant", "functions", "likelihood"
   )
   block <- match.arg(block, vblocks)
   vpositions <- c("start", "end")
@@ -97,7 +110,35 @@ stanvar <- function(x = NULL, name = NULL, scode = NULL,
           "manually via argument 'scode'."
         )
       }
-      scode <- paste0("  ", scode, ";")
+      scode <- paste0(scode, ";")
+    }
+    if (is.null(pll_args)) {
+      # infer pll_args from x
+      if (is.integer(x)) {
+        if (length(x) == 1L) {
+          pll_type <- "int"
+        } else {
+          pll_type <- "int[]"
+        }
+      } else if (is.vector(x)) {
+        if (length(x) == 1L) {
+          pll_type <- "real"
+        } else {
+          pll_type <- "vector"
+        }
+      } else if (is.array(x)) {
+        if (length(dim(x)) == 1L) {
+          pll_type <- "vector"
+        } else if (is.matrix(x)) {
+          pll_type <- "matrix"
+        }
+      }
+      if (!is.null(pll_type)) {
+        pll_args <- paste0(pll_type, " ", name)
+      } else {
+        # don't throw an error because most people will not use threading
+        pll_args <- character(0)
+      }
     }
   } else {
     x <- NULL
@@ -108,11 +149,13 @@ stanvar <- function(x = NULL, name = NULL, scode = NULL,
     if (is.null(scode)) {
       stop2("Argument 'scode' is required if block is not 'data'.")
     }
+    scode <- as.character(scode)
+    pll_args <- as.character(pll_args)
   }
   if (position == "end" && block %in% c("functions", "data", "model")) {
     stop2("Position '", position, "' is not sensible for block '", block, "'.")
   }
-  out <- nlist(name, sdata = x, scode, block, position)
+  out <- nlist(name, sdata = x, scode, block, position, pll_args)
   structure(setNames(list(out), name), class = "stanvars")
 }
 
@@ -139,7 +182,20 @@ collapse_stanvars <- function(x, block = NULL, position = NULL) {
   if (!length(x)) {
     return("")
   }
-  collapse(ulapply(x, "[[", "scode"), "\n")
+  collapse(wsp(nsp = 2), ulapply(x, "[[", "scode"), "\n")
+}
+
+# collapse partial lpg-lik code provided in a stanvars object
+collapse_stanvars_pll_args <- function(x) {
+  x <- validate_stanvars(x)
+  if (!length(x)) {
+    return(character(0))
+  }
+  out <- ulapply(x, "[[", "pll_args")
+  if (!length(out)) {
+    return("")
+  }
+  collapse(", ", out)
 }
 
 # validate 'stanvars' objects
