@@ -33,21 +33,23 @@ test_that("brm produces expected errors", {
                "The term 'x:cs(g)' is invalid")
   expect_error(brm(y~me(x, 2 * g)*me(x, g), dat),
                "Variable 'x' is used in different calls to 'me'")
+  expect_error(brm(y ~ 1 + set_rescor(TRUE), data = dat),
+               "Function 'set_rescor' should not be part")
   
   # autocorrelation
-  expect_error(brm(y ~ 1, dat, autocor = cor_ar(~x+y|g)), 
-               "Autocorrelation structures may only contain 1 time variable")
-  expect_error(brm(y ~ 1, dat, autocor = cor_ar(x~t1|g1)), 
-               "Autocorrelation formulas must be one-sided")
-  expect_error(brm(y ~ 1, dat, autocor = cor_ar(~1|g1/g2)), 
-               paste("Illegal grouping term 'g1/g2'"))
-  expect_error(brm(y ~ 1, dat, poisson(), autocor = cor_ma(~x)),
+  expect_error(brm(y ~ ar(x+y, g), dat), 
+               "Cannot coerce 'x \\+ y' to a single variable name")
+  expect_error(brm(y ~ ar(gr = g1/g2), dat), 
+               "Illegal grouping term 'g1/g2'")
+  expect_error(brm(y ~ ma(x), dat, poisson()),
                "Please set cov = TRUE")
+  expect_error(brm(bf(y ~ 1) + arma(x), dat),
+               "Autocorrelation terms can only be specified")
   
   # ordinal models
   expect_error(brm(rating ~ treat + (cs(period)|subject),
                    data = inhaler, family = categorical()), 
-               "Category specific effects require families")
+               "Category specific effects are not supported")
   
   # families and links
   expect_error(brm(y ~ x, dat, family = gaussian("logit")), 
