@@ -622,18 +622,28 @@ test_that("make_standata handles multi-membership models", {
 
 test_that("by variables in grouping terms are handled correctly", {
   gvar <- c("1A", "1B", "2A", "2B", "3A", "3B", "10", "100", "2", "3")
+  gvar <- rep(gvar, each = 10)
+  g_order <- order(gvar)
+  byvar <- c(0, 4.5, 3, 2, "x 1")
+  byvar <- factor(rep(byvar, each = 20))
   dat <- data.frame(
     y = rnorm(100), x = rnorm(100),
-    g = rep(gvar, each = 10),
-    z = factor(rep(c(0, 4.5, 3, 2, "x 1"), each = 20)),
-    z2 = factor(1:2)
+    g = gvar,
+    g2 = gvar[g_order],
+    z = byvar,
+    z2 = byvar[g_order],
+    z3 = factor(1:2)
   )
   sdata <- make_standata(y ~ x + (x | gr(g, by = z)), dat)
   expect_equal(sdata$Nby_1, 5)
   expect_equal(sdata$Jby_1, as.array(c(2, 2, 1, 1, 5, 4, 4, 5, 3, 3)))
   
-  expect_error(make_standata(y ~ x + (1|gr(g, by = z2)), dat),
-               "Some levels of 'g' correspond to multiple levels of 'z2'")
+  sdata <- make_standata(y ~ x + (x | mm(g, g2, by = cbind(z, z2))), dat)
+  expect_equal(sdata$Nby_1, 5)
+  expect_equal(sdata$Jby_1, as.array(c(2, 2, 1, 1, 5, 4, 4, 5, 3, 3)))
+  
+  expect_error(make_standata(y ~ x + (1|gr(g, by = z3)), dat),
+               "Some levels of 'g' correspond to multiple levels of 'z3'")
 })
 
 test_that("make_standata handles calls to the 'poly' function", {
@@ -805,6 +815,13 @@ test_that("make_standata includes data of special priors", {
   sdata <- make_standata(y ~ x1*x2, data = dat, prior = set_prior(hs))
   expect_equal(sdata$hs_scale_global, 0.1 / sqrt(nrow(dat)))
   
+  # R2D2 prior
+  sdata <- make_standata(y ~ x1*x2, data = dat,
+                         prior = prior(R2D2(0.5, 10)))
+  expect_equal(sdata$R2D2_mean_R2, 0.5)
+  expect_equal(sdata$R2D2_prec_R2, 10)
+  expect_equal(sdata$R2D2_cons_D2, as.array(rep(1, 3)))
+  
   # lasso prior
   sdata <- make_standata(y ~ x1*x2, data = dat,
                          prior = prior(lasso(2, scale = 10)))
@@ -968,3 +985,20 @@ test_that("information for threading is handled correctly", {
   sdata <- make_standata(y ~ 1, dat, threads = threading(2, grainsize = 3))
   expect_equal(sdata$grainsize, 3)
 })
+
+test_that("variables in data2 can be used in population-level effects", {
+  dat <- data.frame(y = 1:10, x1 = rnorm(10), x2 = rnorm(10), x3 = rnorm(10))
+  foo <- function(..., idx = NULL) {
+    out <- cbind(...)
+    if (!is.null(idx)) {
+      out <- out[, idx, drop = FALSE]
+    }
+    out
+  }
+  sdata <- make_standata(y ~ foo(x1, x2, x3, idx = id), data = dat, 
+                         data2 = list(id = c(3, 1)))
+  target <- c("Intercept", "foox1x2x3idxEQidx3", "foox1x2x3idxEQidx1")
+  expect_equal(colnames(sdata$X), target)
+  expect_equivalent(sdata$X[, 2], dat$x3)
+  expect_equivalent(sdata$X[, 3], dat$x1)
+})
